#include <iostream>
#include <cstdlib>
#include <vector>
#include <stack>
#include <cmath>

using namespace std;


#define PRINT

/*
Constants :
q -> number of state
n -> neighborhood size
N=q^(n-1) -> number of state in the graph
m=q^n -> number of edges
*/

int q,n,N,options,iter;
int conf_size;

vector<int> automaton;
vector<int> config,aux;



#define int_pow(a,b) ((int)pow((double)a,(double)b))

void print_usage(){
  cout<<"Usage : ./print RandSeed NumberOfIteration ConfSize NumberOfState NeighborhoodSize Automaton"<<endl;
}


void print_control(){
}

void print_vector(vector<int>* vec){
  for (int i=0;i<(int)(*vec).size();i++){
    cerr<<(*vec)[i];
  }
  cerr<<endl;
}

void print_conf(vector<int>* vec){
	vector<int>::iterator it;
	for(it=(*vec).begin();it!=(*vec).end();it++){
		cout<<*it<<" ";
	}
	cout<<endl;
}


int randbase(){
  double aux=(double) rand();
  return (int) aux;
}



bool string_eq(int l,char *a,char *b){
  for(int i=0;i<l;i++){
    if(!a[i]==b[i]){
      return false;
    }
  }
  return true;
}

int modulo(int n,int q){
  if(n%q<0)
    return q+n%q;
  else
    return n%q;
}

void create_config(){
  config.clear();
	for(int i=0;i<conf_size;i++){
    config.push_back(modulo(randbase(),q));
  }
}


void init(int a,int b){
  q=a;
  n=b;
  N=int_pow(a,b-1);
  create_config();
  aux.resize(conf_size);
}

void create_automaton(char *string){
	automaton.clear();
  int i=0;
  while(string[i]!='\0'){
    i++;
  }
	while(i>0){
    i--;
    automaton.push_back((int) string[i]-'0');
  }
}

void get_opt(int argc, char **argv){
  if(argc<3)
    print_usage();
  else{
    for(int i=1;i<argc-3;i++){
      if(string_eq(7,argv[i],(char*) "--print") || string_eq(2,argv[i],(char*) "-p"))
        options+=1;
    }
		conf_size=atoi(argv[argc-4]);
    init(atoi(argv[argc-3]),atoi(argv[argc-2]));
    iter=atoi(argv[argc-5]);
		create_automaton(argv[argc-1]); // beurk
  }
}

int decimal(int base,int start,int end,vector<int> *vec){
  int res=0;
  for(int i=end;i>=start;i--){
    res=res*base+(*vec)[i];
  }
  return res;
}


void exec(vector<int> *input,vector<int> *output){
  int local=0;
	int s=(int) (*input).size();
  for(int i=(int) (*input).size()-1;i>=(int) (*input).size()-n;i--){
    local=local*q+(*input)[i];
  }
  for(int i=0;i<s;i++){
    local=(local/q)+(*input)[i]*N;
    (*output)[modulo((i-n/2),s)]=automaton[local];
	}
}

void swap(vector<int> **a,vector<int> **b){
  vector<int>** aux=a;
  *a=*b;
  *b=*aux;
}

bool test_id(vector<int> *input, vector<int> *output){
  bool res;
  int i=0;
  while( (*input)[i] == (*output)[i]  && i<(int) (*input).size()){
    i++;
  }
  res=(i==(int) (*input).size());
  return res;
}

bool test_shift(int decalage,vector<int> *input, vector<int> *output){
  int size=(int) (*input).size();
  int i=0;
  while( (*input)[i] == (*output)[modulo(i+decalage,size)]  && i<size){
    i++;
  }
  return (i!=size);
}



int main(int argc, char **argv){
  vector<int>* input;
  vector<int>* output;
  srand((int) argv[1]);
  get_opt(argc,argv);
  input=&config;
  output=&aux;
  cout<<"P2\n";
  cout<<conf_size<<" "<<iter<<endl;
  cout<<q-1<<endl;
	for(int i=0;i<iter;i++){
    print_conf(input);
    exec(input,output);
		print_conf(output);
    exec(output,input);
}
  print_conf(input);
  return 0;
}
