#include <iostream>
#include <stdio.h>
#include <cstdlib>
#include <vector>
#include <algorithm>
#include <cmath>
//#include <map>

using namespace std;


/*
Constants :
q -> number of state
n -> neighborhood size
N=q^(n-1) -> number of state in the graph
m=q^n -> number of edges
*/

int q,n,N,options;

string automaton_string;
vector<int> automaton;
vector<vector<int> > graph;
vector<int> color;
vector<vector<int> > bruijn;

#define int_pow(a,b) ((int)pow((double)a,(double)b))
#define edge(v,x) (v*q+x)
#define dest(v,x) ((v*q+x)%N)
#define succ(v,x) (graph[v*q+x])
#define arrow(v,x) (N*v+x)
#define collapse(a,b) (a==b?0:N*a+b)


void print_automaton(){
	vector<int>::iterator it=automaton.end();
	while(it!=automaton.begin()){
		it--;
		cout<<*it;
	}
}

void print_usage(){
  cout<<"Usage : ./reversibility [-p] NumberOfState NeighborhoodSize Automaton"<<endl;
}

void print_control(){
  cerr<< "State: "<<q<<" | Neighborhood : "<<n<<" | BruijnSize :"<<N<<" | Automaton : ";
	print_automaton();
	cerr<<endl;
}

void init(int a,int b){
  q=a;
  n=b;
  N=int_pow(a,b-1);
	color.resize(N*N,0);
}

void create_automaton(string *string){
  automaton.clear();
  int i=0;
  while((*string)[i]!='\0'){
    i++;
  }
  while(i>0){
    i--;
    automaton.push_back((int) (*string)[i]-'0');
  }
}


/*********************************************************************
 *                      Creation of the graph
 *********************************************************************/


void make_bruijn(){
	bruijn.clear();
	bruijn.resize(N*q);
	for(int i=0;i<N;i++){
		for(int j=0;j<q;j++){
//			cerr<<i<<j<<endl;
			bruijn[edge(i,automaton[edge(i,j)])].push_back(dest(i,j));
		}
	}
}

void clean_diag(){
	vector<int>::iterator it=graph[0].begin();
	vector<int>::iterator itb;
	sort(it,graph[0].end());
	while(*it == 0 && it!=graph[0].end())
			graph[0].erase(it);
	if(it!=graph[0].end()){
		itb=it;
		itb++;
		while(itb!=graph[0].end()){
			if(*it==*itb)
				graph[0].erase(itb);
			else{
			it++;
			itb++;
			}
		}
	}
}


void make_graph(){
	graph.clear();
	graph.resize(N*N);
	vector<int>::iterator itera;
	vector<int>::iterator iterb;
	for(int i=0;i<N;i++){
		for(int j=0;j<N;j++){
			for(int k=0;k<q;k++){
				for(itera=(bruijn[edge(i,k)]).begin();itera!=(bruijn[edge(i,k)]).end();itera++){
					for(iterb=(bruijn[edge(j,k)]).begin();iterb!=(bruijn[edge(j,k)]).end();iterb++){
						graph[collapse(i,j)].push_back(collapse(*itera,*iterb));
					}
				}
			}
		}
	}
	clean_diag();
}

void print_base(int n){
	string chain=(n==0?"0":"");
	while(n>0){
		chain.push_back('0'+(n%q));
		n=n/q;
	}
	reverse<std::string::iterator>(chain.begin(),chain.end());
	cerr<<chain;
}

void print_state(int i){
		print_base(i/N);
		cerr<<"*";
		print_base(i%N);
}


void print_graph(){
	for(int i=0;i<N*N;i++){
		cerr<<"State : ";
		print_state(i);
		cerr<<" - Adj : ";
		for(int j=0;j<(int) graph[i].size();j++){
			print_state(graph[i][j]);
			cerr<<", ";
		}
	cerr<<endl;
	}
}


/*********************************************************************
 *                      Detection of cycle
 ********************************************************************/

bool dfs_aux(int v){
	color[v]=1;
	for(int i=0;i<(int) graph[v].size();i++){
		switch(color[graph[v][i]]){
		case 1:
			if((options>>1)&1){
				cerr<<"Cycle :";
				print_state(graph[v][i]);
				cerr<<" <- ";
				print_state(v);
			}
			return false;
		case 0:
			if(!dfs_aux(graph[v][i])){
				if((options>>1)&1){
					cerr << " <- ";
					print_state(v);
				}
				return false;
			}
		}
	}
	color[v]=2;
	return true;
}

bool dfs(){
 	for(int i=1;i<N*N;i++){
		if(color[i]==0){
			if(!dfs_aux(i)){
				if((options>>1)&1)
					cerr<<endl;
				return false;
			}
		}
	}
	return true;
}

bool string_eq(int l,char *a,char *b){
  for(int i=0;i<l;i++){
    if(a[i]!=b[i]){
      return false;
    }
  }
  return true;
}

void get_opt(int argc, char **argv){
}


void create_automat(){
	int size=(automaton_string).size();
	reverse(automaton_string.begin(),automaton_string.end());
	automaton.resize(size);
	for(int i=0;i<size;i++){
		automaton[i]=(int) automaton_string.at(i)-'0';
	}
}


int main(int argc, char **argv) {
	FILE* infile;
	char c;
  if(argc<3)
    print_usage();
  else{
		for(int i=1;i<argc-3;i++){
			if(string_eq(7,argv[i],(char*) "--print") || string_eq(2,argv[i],(char*) "-p"))
				options+=1;
			if(string_eq(7,argv[i],(char*) "--debug") || string_eq(2,argv[i],(char*) "-d"))
				options+=2;
		}
		init(atoi(argv[argc-3]),atoi(argv[argc-2]));
		infile=fopen(argv[argc-1],"r");
		while((c=fgetc(infile))!=EOF){
			if(c=='\n'){
				reverse(automaton.begin(),automaton.end());
				color.clear();
				color.resize(N*N,0);
				make_bruijn();
				make_graph();
				if(dfs()){
					if(options&1)
						cerr<<"Injective\n";
					print_automaton();
					cout<<endl;
				}
				else{
					if(options&1)
						cerr<<"Non-injective\n";
					}
				automaton.clear();
			}
			else
				automaton.push_back((int)c-'0');
		}
		fclose(infile);
	}
	return 0;
}
