#include <iostream>
#include <cstdlib>
#include <vector>
#include <stack>
#include <cmath>

using namespace std;


#define PRINT

/*
Constants :
q -> number of state
n -> neighborhood size
N=q^(n-1) -> number of state in the graph
m=q^n -> number of edges
*/

int q,n,N,m,solutions,options;

vector<int> sorted;
vector<int> label;
vector<vector <int> > potential;


#define int_pow(a,b) ((int)pow((double)a,(double)b))

#define edge(v,x) (v*q+x)
#define succ(v,x) ((v*q+x)%N)
#define pred(v,x) ((v/q)+x*(N/q))
#define head(e) (e%N)
#define tail(e) (e/q)
#define letter(v) (v/N)
#define last(v) (v%q)



void print_usage(){
  cout<<"Usage : ./scca [Options] NumberOfState NeighborhoodSize"<<endl<<endl;
	cout<<"Options:" <<endl;
	cout<<"  -p  --print\t   Print the solutions"<<endl;
	cout<<"  -v  --verbose\t   Give more informations"<<endl;
}


void print_control(){
  cerr<< "State: "<<q<<" | Neighborhood : "<<n<<" | Vertex="<<N<<" | Edges="<<m<<" | #CA("<<q<<","<<n<<")="<<q<<"^"<<int_pow(q,n)<<endl;
}

void print_vector(vector<int>* vec){
  for (int i=0;i<(int)(*vec).size();i++){
    cerr<<" "<<(*vec)[i];
  }
  //	cerr<<endl;
}

void print_reached(vector<bool>* vec){
  for (int i=0;i<(int)(*vec).size();i++){
    if((*vec)[i])
      cerr<<" "<<i;
  }
  //	cerr<<endl;
}

vector<bool> sort_reached;
vector<bool> edge_marked;

void sort_dfs(int v){
  if(sort_reached[v])
    return;
  else{
    sort_reached[v]=true;
    for(int i=0;i<q;i++){
      sorted.push_back(edge(v,i));
      sort_dfs(succ(v,i));
    }
  }
}

void sort_opt(int v){
  if(sort_reached[v])
    return;
  else{
    sort_reached[v]=true;
    for(int i=0;i<q;i++){
      if(sort_reached[succ(v,i)]){
        sorted.push_back(edge(v,i));
        edge_marked[edge(v,i)]=true;
      }
      if(sort_reached[pred(v,i)] && !edge_marked[edge(pred(v,i),last(v))]){
        sorted.push_back(edge(pred(v,i),last(v)));
        edge_marked[edge(pred(v,i),last(v))]=true;
      }
    }
    for(int i=0;i<q;i++){
      if(!edge_marked[edge(v,i)]){
        sorted.push_back(edge(v,i));
        edge_marked[edge(v,i)]=1;
        sort_opt(succ(v,i));
      }
    }
  }
}

#define OPT

void sort_edges(){
  sort_reached.resize(N,false);
  edge_marked.resize(m,false);
  sorted.clear();
#ifdef OPT
  sort_opt(0);
#else
  sort_dfs(0);
#endif
}



vector<bool> reached;

void init(int a,int b){
  q=a;
  n=b;
  N=(int) pow((double)q,(double)(n-1));
  m=N*q;
  reached.resize(N,false);
  potential.resize(N,vector<int>(q,0));
  //	potential[0][0]=1;
  label.resize(m,0);
}

void print_solution(){
  for(int i=m-1;i>=0;i--){
    cout<<label[i];
  }
  cout<<endl;
}

int iter;


int compare_vector(int h, int t, int l){
  int res=-1;
  for(int i=0;i<l;i++){
    if(potential[t][i]==potential[h][i])
      continue;
    else if(potential[h][i]==potential[t][i]+1){
      if(res==-1)
        res=i;
      else
        return -1;
    }
    else
      return -1;
  }
  if(potential[h][l]==potential[t][l]){
    if(res==-1)
      res=l;
    else
      return -1;
  }
  else if(!(potential[h][l]==potential[t][l]-1))
    return -1;
  for(int i=l+1;i<q;i++){
    if(potential[t][i]==potential[h][i])
      continue;
    else if(potential[h][i]==potential[t][i]+1){
      if(res==-1)
        res=i;
      else
        return -1;
    }
    else return -1;
  }
  return res;
}

void print_edge(int t,int h,int e){
#ifdef DEBUG
  print_vector(&potential[t]);
  cerr<<" -> ";
  print_vector(&potential[h]);
  cerr<< " | l="<<letter(e)<<endl;
#endif
}

void dfs(int ce){
  iter++;
  int e=sorted[ce];
  if(ce==m){
		if(options&1)
      print_solution();
    ++solutions;
    return;
  }
  int t,h,tmp;
  t=tail(e);
  h=head(e);
  if(reached[h]){
    tmp=compare_vector(h,t,letter(e));
    if (tmp==-1){
      return;
    }
    else{
      label[e]=tmp;
      dfs(ce+1);
    }
  }
  else{
    reached[h]=true;
    for(int i=0; i<q;i++){
      for(int j=0;j<q;j++)
        potential[h][j]=potential[t][j];
      label[e]=i;
      potential[h][i]++;
      potential[h][letter(e)]--;
      dfs(ce+1);
    }
    reached[h]=false;
  }
}

bool string_eq(int l,char *a,char *b){
  for(int i=0;i<l;i++){
    if(a[i]!=b[i]){
      return false;
    }
  }
  return true;
}

void get_opt(int argc, char **argv){
  if(argc<3)
    print_usage();
  else{
    for(int i=1;i<argc-2;i++){
      if(string_eq(7,argv[i],(char*) "--print") || string_eq(2,argv[i],(char*) "-p"))
        options+=1;
      if(string_eq(7,argv[i],(char*) "--verbose") || string_eq(2,argv[i],(char*) "-v"))
        options+=2;
    }
    init(atoi(argv[argc-2]),atoi(argv[argc-1]));
  }
}


int main(int argc, char **argv) {
  int aux=0;
  if(argc<3)
    print_usage();
  else{
    get_opt(argc, argv);
    if((options>>1)&1)
      print_control();
    reached[0]=true;
    iter=0;
    sort_edges();
    dfs(1);
    if(options&1)
      cerr<<solutions<<" solutions"<<endl;
    else
      cout<<solutions<<" solutions"<<endl;
    if((options>>1)&1){
      n=iter;
      while(iter>0){
        iter=iter/q;
        aux++;
      }
      cerr<<"Less than "<<q<<"^"<<aux<<" iterations ("<<n<<" exactly)"<<endl;
    }
      //, soit "<<iter/(pow((double)q,(double)m))<<endl;
  }
  return 0;
}
